#!/usr/bin/python
# -*- coding: UTF-8 -*-
# checksysreport, a tool to spot unsupported issues.
# Copyright 2005 Pierre Amadio 
# pamadio@redhat.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
Expl:

import sys
sys.path.append("/usr/share/checksysreport")
import checksysreport_wrapper
dir="/home/pamadio/mgmt1"
test=checksysreport_wrapper.checksysreport_wrapper(dir)

print test.checker.getRunningKernel()
print test.generate_report()
"""

import sys
import os
import re
sys.path.append('/usr/share/checksysreport')
sys.path.append('/usr/share/checksysreport/plugins')
import sysreportChecker
import globals
import optparse


class checksysreport_wrapper:
    
    def __init__(self,sysreportDir,noChangelog=False,arch=None,zstream=None,fullChangelog=None):

        self.dict =  {'pluginOpt': None,
                 'listPackages': None,
                 'listChannels': None,
                 'sysreportDir': sysreportDir,
                 'pidToDiff': None,
                 'noChangelog': noChangelog,
                 'listAllPackages': None,
                 'arch': arch,
                 'zstream': zstream,
                 'fullChangelog': fullChangelog
                 }
        #create the object that checksysreport expects
        self.options = optparse.Values(self.dict)
        
        release=globals.detect_release(sysreportDir)

        globals.sosReportCompat(sysreportDir)
        releaseDetected=0



        if release=='2.1':
            checker=sysreportChecker.sysreportChecker_2(sysreportPath=sysreportDir,options=self.options)
            releaseDetected=1
        if release=='3':
            checker=sysreportChecker.sysreportChecker_3(sysreportPath=sysreportDir,options=self.options)
            releaseDetected=1
        if release=='4':
            checker=sysreportChecker.sysreportChecker_4(sysreportPath=sysreportDir,options=self.options)
            releaseDetected=1

        if release=='5':
            checker=sysreportChecker.sysreportChecker_5(sysreportPath=sysreportDir,options=self.options)
            releaseDetected=1


        if releaseDetected==0:
            globals.die("Unable to detect Red Hat Release Version")

        self.checker=checker


    
    def generate_report(self):
        plugin_dic={}

        for cur_plug in globals.config["mandatory_plugin"]:
            if cur_plug not in plugin_dic.keys():
                plugin_dic[cur_plug]=[]
            for cur_dic in globals.config['mandatory_plugin'][cur_plug]:
                plugin_dic[cur_plug].append(cur_dic)


        
        return globals.generate_report(self.checker,self.options,plugin_dic)

    def getChecker(self):
        return self.checker
