#!/usr/bin/python
# -*- coding: UTF-8 -*-
# rhnapiwrap : a python class to handle rhn api call with a bit of caching
# Copyright 2005 Pierre Amadio 
# pamadio@redhat.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import xmlrpclib
import re

class rhnapiwrap:
    "A class to handle rhn calls"
    #https://rhn.redhat.com/rpc/api/
    server_url=''
    server = ''
    userLogin='none';
    userPassword='none';
    timeLenght=1200;
    base_channel_cache={}
    entitlement_cache={}
    child_channels_cache={}
    packages_cache={}
    software_channels_cache=[]
    packages_by_channels_cache={}
    latest_packages_by_channels_cache={}
    files_in_package_cache={}
    changelog_cache={}
    session='';
    
    def __init__(self,login,password,server_url='https://rhn.redhat.com/rpc/api'):
        #print "rhn initialisation"
        self.userLogin=login
        self.userPassword=password
        self.server_url=server_url
        self.server=xmlrpclib.Server(self.server_url)
        """
        if login:
            self.login()
        """
        
    def login(self):
        #print "Let's login with ",self.userLogin," ",self.userPassword
        self.base_channel_cache={}
        self.entitlement_cache={}
        self.child_channels_cache={}
        self.packages_cache={}
        self.software_channels_cache=[]
        packages_by_channels_cache={}
        latest_packages_by_channels_cache={}
        
        try:
            result = self.server.auth.login(self.userLogin,self.userPassword,self.timeLenght)
            self.session=result
            return 0
        except xmlrpclib.Fault, fault:
            #print "trouble at login",fault
            raise
            
    def list_user_system(self):
        #print "list_user_system"
        try:
            result=self.server.system.list_user_systems(self.session)
            return result
        except xmlrpclib.Fault, fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_user_system()
            else:
                raise

    def get_user_detail(self,login):
        #print "Getting detail for:",login
        try:
            result=self.server.user.get_details(self.session,login)
            return result
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.get_user_detail(login)
            else:
                raise
            
            
    def get_entitlement(self,id):
        #print "get_entitlement for",id
        if(self.entitlement_cache.has_key(id)):
            return self.entitlement_cache[id]
        try:
            result=self.server.system.get_entitlement(self.session,id)
        except xmlrpclib.Fault, fault:
            if fault.faultCode==-20:
                self.login()
                return self.get_entitlement(id)
            else:
                raise
        self.entitlement_cache[id]=result
        return result

    def list_groups(self,sid):
        #print "list_groups for",sid
        try:
            result=self.server.system.list_groups(self.session,sid)
            return result
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_groups(sid)
            else:
                raise
       
    def list_users(self):
        #print "list_users"
        try:
            result=self.server.user.list_users(self.session)
            return result
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_users()
            else:
                raise
       
    def list_roles(self,login):
        #print "Listing roles for",login
        try:
            result=self.server.user.list_roles(self.session,login)
            return result
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_roles(login)
            else:
                raise
            
    def list_base_channels(self,sid):
        #print "list_base_channels for ",sid
        sid=int(sid)
        if(self.base_channel_cache.has_key(sid)):
            return self.base_channel_cache[sid]
        try:
            result=self.server.system.list_base_channels(self.session,sid)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_base_channels(sid)
            else:
                raise
        self.base_channel_cache[sid]=result
        return result
        

    def list_current_base_channel(self,sid):
        #print "list_current_base_channel for ",sid
        base=self.list_base_channels(sid)
        for c in base:
            if c['CURRENT_BASE']==1:
                return c['ID']
        return 0    



    # This gives a list of subscribable child channels :
    # it s a list of all child channels we sid is not hooked to.
    # but that could be used if needed.
    def list_child_channels(self,sid):
        #print "Listing child channels for ",sid
        if(self.child_channels_cache.has_key(sid)):
            return self.child_channels_cache[sid]
        try:
            result=self.server.system.list_child_channels(self.session,sid)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_child_channels(sid)
            else:
                raise
        self.child_channels_cache[sid]=result
        return result
   

    def list_software_channels(self):
        #print "Listing all software_channels"
        if(len(self.software_channels_cache)):
            return self.software_channels_cache
        
        try:
            result=self.server.channel.list_software_channels(self.session)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_software_channels()
            else:
                raise
        self.software_channels_cache=result    
        return result


            
    def list_packages(self,sid):
        #print "Getting packages list for ",sid
        if(self.packages_cache.has_key(sid)):
           return self.packages_cache[sid]
        try:
            result=self.server.system.list_packages(self.session,sid)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_packages(sid)
            else:
                raise
        self.packages_cache[sid]=result
        return result

    def list_child_channels_of(self,label):
        #print "list_child_channels_of",label
        #gives a list of label of channels sons of the channel labbeled as the 'label' argument.
        output=[];
        totalChans=self.list_software_channels()
        for c in totalChans:
            curLabel=c['channel_label']
            fatherLabel=c['channel_parent_label']
            if fatherLabel==label:
                output.append(curLabel)
        return output        



    def get_errata_details(self,advisory):
        #print "fetching details for errata",advisory
        output={}
        try:
            output=self.server.errata.get_details(self.session,advisory)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.get_errata_details(advisory)
            else:
                raise
        return output    

    def list_affected_systems(self,advisory):
        #print "Listing affected system fo errata",advisory
        output={}
        try:
            output=self.server.errata.list_affected_systems(self.session,advisory)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_affected_systems(advisory)
            else:
                raise
        return output    


        
    def errata_list_packages(self,advisory):
        #print "Listing package fo errata",advisory
        #valid advisory=RHSA-2004:600
        output={}
        try:
            output=self.server.errata.list_packages(self.session,advisory)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.errata_list_packages(advisory)
            else:
                raise
        return output    

    #Channel Software calls.


    def list_packages_in_channel_software(self,channel_label):
        #print "list_packages_in_channel_software(self,channel_label)",channel_label
        if(self.packages_by_channels_cache.has_key(channel_label)):
            return self.packages_by_channels_cache[channel_label]
        output=[]
        try:
            output=self.server.channel.software.list_all_packages(self.session,channel_label)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_packages_in_channel_software(channel_label)
            elif fault.faultCode==-210:
                """210: No such channel"""
                return []
            else:
                raise

        self.packages_by_channels_cache[channel_label]=output
        return output    

           
    
    def list_latest_packages_in_channel_software(self,channel_label):
        if(self.latest_packages_by_channels_cache.has_key(channel_label)):
            return self.latest_packages_by_channels_cache[channel_label]
        output=[]
        try:
            output=self.server.channel.software.list_latest_packages(self.session,channel_label)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_latest_packages_in_channel_software(channel_label)
            elif fault.faultCode==-210:
                """210: No such channel"""
                return []
            else:
                raise
        self.latest_packages_by_channels_cache[channel_label]=output
        return output    

                   
    def list_filesInfo_in_package(self,pid):
        """
         Return a list of file and meta info in a given package.
        """
        out=[]
        
        try:
            out=self.server.packages.list_files(self.session,pid)
            return out
        except xmlrpclib.Fault, fault:
            if fault.faultCode==-20:
                self.login()
                return self.list_filesInfo_in_package(pid)
            else:
                raise


    def get_changelog(self,pid):
        """
          Return a dictionnary describing the changelog for package with id pid.
          key is a date of the change in YYYYMMDD format so it is easily
          sortable.

          value is another dictionnary with the following keys:
          date: date of the change as it appears on the web site
          header: some informations about who applyed the change
          channges: a description of the change.


        """
        pid=str(pid)
        if (self.changelog_cache.has_key(pid)):
            return self.changelog_cache[pid]

        rawOutput=[]
        try:
            rawOutput=self.server.packages.list_changelog(self.session,int(pid))
            #print "rawOutput",rawOutput
        except xmlrpclib.Fault,fault:
            print "fault=",fault
            if fault.faultCode==-20:
                self.login()
                return self.get_changelog(pid)
            if fault.faultCode==-2:
                """
                adk's session issue
                <Fault -2: 'unhandled internal exception: Could not find session with id: 3897268229'>
                """
                self.login()
                return self.get_changelog(pid)
            else:
                raise
        """
        Oki, now we have rawOuput looking like :
        
        [{'entry_author': 'Tim Powers <timp@redhat.com>', 'entry_text': '- rebuilt', 'entry_date': '2003-01-22 22:00:00'}, {'en ...]
        
        We got to return something like
        {
         20050405:{'date':'05-APR-05','header':'Dave Jones\ndavej@redhat.com:','changes':'- Fix tmpfs truncate BUG().\n-yet another fix...'}
         20050408:{'date':'08-APR-05','header':'Dave Jones\ndavej@redhat.com:','changes':'- another entry decribing fix.\n-yet another fix...'}
         ....
         ....
        }
       
        """
        output={}
        for entry in rawOutput:
            entryAuthor=entry['entry_author'].encode('utf8')
            entryText=entry['entry_text'].encode('utf8')
            entryDate=entry['entry_date'].encode('utf8')
            pattern=r"^(\d{4})-(\d{2})-(\d{2}) \d{2}:\d{2}:\d{2}"
            regSearch=re.compile(pattern).search(entryDate)
            if regSearch:
                newDate=regSearch.group(1)+regSearch.group(2)+regSearch.group(3)
            else:
                #print "Wrong entry_date un changelog for package"+str(pid)+" :"+entryDate
                newDate=19710410
            output[newDate]={}
            output[newDate]['date']=entryDate
            output[newDate]['header']=entryAuthor
            output[newDate]['changes']=entryText
            
        self.changelog_cache[pid]=output
        return output


    def get_bugs_for_errata(self,advisory):
        """
         Return a list of information related to bugs solved by a given errata advisory.
         cf https://rhn.redhat.com/rpc/api/errata/bugzilla_fixes/
        """
        try:
            result=self.server.errata.bugzilla_fixes(self.session,advisory)
            return result
        except xmlrpclib.Fault, fault:
            if fault.faultCode==-20:
                self.login()
                return self.get_bugs_for_errata(advisory)
            else:
                raise


        
    def listErrata(self,channelLabel):
        """ List the errata applicable to a channel
        https://rhn.redhat.com/rhn/apidoc/channel/software/ChannelSoftwareHandler.jsp#listErrata
        """

        try:
            result=self.server.channel.software.listErrata(self.session,channelLabel)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.listErrata(channelLabel)
            elif fault.faultCode==-210:
                """210: No such channel"""
                return []
            else:
                raise
        return result



    def listCves(self,advisoryName):
        """
        List cves according to an advisory.
        https://rhn.redhat.com/rhn/apidoc/errata/ErrataHandler.jsp#listCves
        """
        try:
            result=self.server.errata.listCves(self.session,advisoryName)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.listCves(advisoryName)
            else:
                raise
        return result

    def listDependencies(self,pid):
        #print "yo,",pid
        """
        List dependencies for package pid.
        https://rhn.redhat.com/rhn/apidoc/packages/PackagesHandler.jsp#listDependencies
        """
        
        try:
            result=self.server.packages.listDependencies(self.session,pid)
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-20:
                self.login()
                return self.listDependencies(pid)
            else:
                raise
        return result
